use sqlx::PgPool;

use crate::models::Token;

use super::PasswordChange;

pub struct PasswordChangeBuilder<'a, T>
where
    T: Token,
{
    pool: &'a PgPool,
    user_id: i32,
    password: &'a str,
    password_retyped: &'a str,
    token: Option<T>,
    current_password: Option<String>,
}

impl<T> PasswordChangeBuilder<'_, T>
where
    T: Token,
{
    pub fn new<'a>(
        pool: &'a PgPool,
        user_id: i32,
        password: &'a str,
        password_retyped: &'a str,
    ) -> PasswordChangeBuilder<'a, T>
    where
        T: Token,
    {
        PasswordChangeBuilder {
            pool,
            user_id,
            password,
            password_retyped,
            token: None,
            current_password: None,
        }
    }

    pub fn with_token(mut self, token: T) -> Self {
        self.token = Some(token);
        self
    }

    pub fn with_current_password(mut self, current_password: &str) -> Self {
        self.current_password = Some(current_password.to_string());
        self
    }

    pub fn build<'a>(&'a mut self) -> PasswordChange<'a, T> {
        PasswordChange {
            pool: self.pool,
            user_id: self.user_id,
            password: self.password,
            password_retyped: self.password_retyped,
            token: self.token.take(),
            current_password: self.current_password.as_deref(),
        }
    }
}
